/*
   Sprendim bei testus pareng Mindaugas Plukas.
   Sprendimo pagrindas yra grafo tranzityvaus udarinio radimo algoritmas,
   pasilytas Floyd'o bei Warshall'o.
   Sprendimo idja yra tokia. Tarkime, kad turime apskaiiuot valiut
   santyki lentel, kai ivestiniams santykiams apskaiiuoti galima naudoti
   tarpines valiutas 1,...,k (k>=0); paymkime i lentel V(k), o jos elementus -
   V(k)[i,j] ( i=1...N,j=1..N, cia N - valiutu kiekis). Tada valiutu santykiu
   lentele V(k+1), kai ivestiniams santykiams apskaiiuoti galima naudoti tarpines
   valiutas 1,...,k+1 , galime apskaiiuoti tokiu bdu:
      V(k+1)[i,j] = V(k)[i,j], jei V(k)[i,j] != 0 ir
      V(k+1)[i,j] = V(k)[i,k+1] * V(k)[k+1,j], jei V(k)[i,j] = 0.

   Taigi , jei V(0) yra duotoji valiut santyki lentel, papildyta santykiais, kuriuos
   galime apskaiiuoti be tarpini valiut pagalbos, tai V(N) bus iekomoji galimai
   pilna valiut santyki lentel.

   Algoritmo operacij skaiius yra apribotas C * N^3, ia C - konstanta(nepriklausanti nuo N).

*/


#include <stdlib.h>
#include <stdio.h>



/*
  Valiut santyki lentel saugome dvimaiame masyve. Jo elementas [i][j]
  ( i=0,...,N-1, j=0,...,N-1 ) nusako kiek valstybs (i+1) valiutos vienetas
  kainuoja valstybs (j+1) valiutos vienet.
*/
#define MAX_VALSTYBIU_SK  120
typedef float TSantykiuLentele[MAX_VALSTYBIU_SK][MAX_VALSTYBIU_SK] ;


int valstSk = 0 ;
TSantykiuLentele valiutuSant ;


void main()
{
   FILE *fpPrDuom = NULL ;
   FILE *fpRezDuom = NULL ;
   int i = 0, j = 0, k = 0 ;


   /* Nuskaitom pradinius duomenis */
   fpPrDuom = fopen( "VALIUTOS.DAT", "r" ) ;
   fscanf( fpPrDuom, " %d ", &valstSk ) ;
   for ( i = 0 ; i < valstSk ; i ++ )
     for ( j = 0 ; j < valstSk ; j ++ )
      {
        fscanf( fpPrDuom, " %f ", &valiutuSant[i][j] ) ;
      }
   fclose( fpPrDuom ) ;


   /* Apskaiiuojam trkstamus tiesioginius(be tarpini valiut) santykius */
   for ( i = 0 ; i < valstSk ; i ++ )
     for ( j = 0 ; j < valstSk ; j ++ )
       if ( valiutuSant[i][j] == 0.0 )
        {
          if ( i == j )
           {
             valiutuSant[i][i] = 1.0 ;
           }
          else
           {
             if ( valiutuSant[j][i] != 0.0 )
               valiutuSant[i][j] = 1/valiutuSant[j][i] ;
           }
        }

   for ( k = 0 ; k < valstSk ; k ++ )
     /* Apskaiiuojam santykius, kuriuos galima apskaiiuoti pasinaudojant  */
     /* tarpinmis valiutomis {1,2,...,k+1}                                 */
    for ( i = 0 ; i < valstSk ; i ++ )
     for ( j = 0 ; j < valstSk ; j ++ )
       if ( valiutuSant[i][j] == 0.0 )
        {
          if ( valiutuSant[i][k] != 0.0 &&
               valiutuSant[k][j] != 0.0 )
            valiutuSant[i][j] = valiutuSant[i][k] * valiutuSant[k][j] ;
        }


   /* Ivedam rezultat */
   fpRezDuom = fopen( "VALIUTOS.REZ", "w" ) ;
   fprintf( fpRezDuom, "%d\n", valstSk ) ;
   for ( i = 0 ; i < valstSk ; i ++ )
    {
      for ( j = 0 ; j < valstSk ; j ++ )
        fprintf( fpRezDuom, "%12f  ", valiutuSant[i][j] ) ;
      fprintf( fpRezDuom, "\n" ) ;
    }
   fclose( fpRezDuom ) ;

}

