#include <cstdio>
#include <cstdlib>

#define ON_EDGE    0
#define INSIDE     1
#define OUTSIDE    2

#define N_MAX 512
#define sgn(x) ((x) > 0 ? 1 : ((x) < 0 ? -1 : 0))

int N, A, B;
int X[N_MAX][3], Y[N_MAX][3];

void get_abc (int x1, int y1, int x2, int y2, int &a, int &b, int &c)
{
    a = y2 - y1;
    b = x1 - x2;
    c = y1 * x2 - x1 * y2;
}

int point_sign (int x1, int y1, int x2, int y2, int _x, int _y)
{
    int a, b, c;
    get_abc(x1, y1, x2, y2, a, b, c);
    return sgn(a * _x + b * _y + c);
}

int point_inside (int n, int x, int y)
{
    int sgn[3], i;
    for (i = 0; i < 3; i++)
        sgn[i] = point_sign(
            X[n][i], Y[n][i], X[n][(i + 1) % 3], Y[n][(i + 1) % 3], x, y
        );
        
    if (sgn[0] * sgn[1] < 0 || sgn[0] * sgn[2] < 0 || sgn[1] * sgn[2] < 0)
        return OUTSIDE;
        
    if (sgn[0] == 0 || sgn[1] == 0 || sgn[2] == 0)
        return ON_EDGE;
        
    return INSIDE;
}

int segment_intersect (int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4)
{
    int a1, b1, c1, a2, b2, c2;
    get_abc(x1, y1, x2, y2, a1, b1, c1);
    get_abc(x3, y3, x4, y4, a2, b2, c2);
    
    return
        sgn(a1 * x3 + b1 * y3 + c1) * sgn(a1 * x4 + b1 * y4 + c1) < 0 &&
        sgn(a2 * x1 + b2 * y1 + c2) * sgn(a2 * x2 + b2 * y2 + c2) < 0;
}

int triangle_intersect (int n1, int n2)
{
    int i, j, x, t1 = 0, t2 = 0;
    for (i = 0; i < 3; i++)
    {
        if ((x = point_inside(n2, X[n1][i], Y[n1][i])) == ON_EDGE)
            t1++;
        if (x == INSIDE)
            return 1;
            
        if ((x = point_inside(n1, X[n2][i], Y[n2][i])) == ON_EDGE)
            t2++;
        if (x == INSIDE)
            return 1;
    }
            
    if (t1 == 3 || t2 == 3)
        return 1;
        
    for (i = 0; i < 3; i++)
        for (j = 0; j < 3; j++)
            if (segment_intersect(
                X[n1][i], Y[n1][i], X[n1][(i + 1) % 3], Y[n1][(i + 1) % 3],
                X[n2][j], Y[n2][j], X[n2][(j + 1) % 3], Y[n2][(j + 1) % 3]
                ))
            {
                //printf("%d %d %d %d, %d %d %d %d\n",
                //X[n1][i], Y[n1][i], X[n1][(i + 1) % 3], Y[n1][(i + 1) % 3],
                //X[n2][j], Y[n2][j], X[n2][(j + 1) % 3], Y[n2][(j + 1) % 3]);
                return 1;
            }
                
    return 0;
}

int solve ()
{
    int i, j, area = 0;
    
    // Each vertex should be inside
    for (i = 0; i < N; i++)
    {
        for (j = 0; j < 3; j++)
            if (point_inside(N, X[i][j], Y[i][j]) == OUTSIDE)
                return 0;
                
        area += abs(
            (X[i][1] * Y[i][2] - X[i][2] * Y[i][1]) -
            (X[i][0] * Y[i][2] - X[i][2] * Y[i][0]) +
            (X[i][0] * Y[i][1] - X[i][1] * Y[i][0])
        );
    }
    
    //printf("Vertexes inside.\n");
                
    // Check area
    if (area != A * B)
        return 0;

    //printf("Area OK.\n");
        
    // Check intersections
    for (i = 0; i < N; i++)
        for (j = i + 1; j < N; j++)
            if (triangle_intersect(i, j))
                return 0;
        
    //printf("Triangles do not intersect.\n");
    return 1;
}

void read_solve ()
{
    int tests, i, j;
    
    freopen("part.in", "r", stdin);    
    freopen("part.out", "w", stdout);
    
    scanf("%d", &tests);
    for (; tests--;)
    {
        scanf("%d%d%d", &A, &B, &N);
        
        for (i = 0; i < N; i++)
            for (j = 0; j < 3; j++)
                scanf("%d%d", &X[i][j], &Y[i][j]);
                
        X[N][0] = 0; Y[N][0] = 0;
        X[N][1] = A; Y[N][1] = 0;
        X[N][2] = 0; Y[N][2] = B;
                
        printf("%d\n", solve());
    }
}

int main ()
{
    read_solve();
    return 0;
}
