#include <stdio.h>
#include <math.h>
#define InFile "petrom.in"
#define OutFile "petrom.out"
#define INFINIT 30000
#define NMAX 1001
#define KMAX 301

int   d[NMAX];
int   cmin[KMAX][NMAX];
int   cost[NMAX][NMAX];
int   k, n;
int   sol[KMAX+1];

void det_cost(void);
void rezolvare(void);
void citire(void);
void afisare(void);


int main()
{
  citire();
  rezolvare();
  afisare();
  return 0;
}

void det_cost()
{
  int i,j,l,med,err;
  /* cost[a][b] = costul de transport pentru cazul in care un depozit
                alimenteaza benzinariile din intervalul [a,b], a<b
   cost[a][a]=0
   costul optim se obtine amplasand un depozit in benzinaria
   situata in mijlocul intervalului */

  for (i=1;i<=n;i++)
      for (j=i+1;j<=n;j++)
      {
	med = d[(i+j)/2];
	err = 0;
	for (l=i; l<=j; l++)
	    err += abs(d[l] - med);
	cost[i][j] = err;
      }
}

void rezolvare()
{
  int i, j, p;
  short back[KMAX][NMAX];
  det_cost();

  /* Metoda: programare dinamica.
  Subproblema:
  Sa se determine o amplasare optima a i depozite in benzinariile 1,..., j

  cmin[i][j]= costul minim de transport in conditiile in care construim
              i depozite in benzinariile 1, ..., j

  Relatia de recurenta:
  cmin[1][j]=cost[1][j], pentru j=1, ..., n
  cmin[i][j]=min {cmin[i-1][p-1]+cost[p][j]| p=i,i+1, ..., j}

  Pentru reconstituirea solutiei vom utiliza o matricea back
  back[i][j]= pozitia p pentru care se obtine costul minim cmin[i][j]

  Solutia problemei initiale se afla in cmin[k][n]

  */


  for (j=1; j<=n; j++)
      {
       cmin[1][j] = cost[1][j];
       back[1][j] = 1;
      }


  for (i=2; i<=k; i++)
       for (j=1; j <= n; j++)
           {
	   cmin[i][j] = INFINIT;
	   for (p=i; p<=j; p++)
	       if (cost[p][j]+cmin[i-1][p-1] < cmin[i][j])
	          {
	           cmin[i][j] = cost[p][j] + cmin[i-1][p-1];
	           back[i][j] = p;
	          }
          }

  sol[k+1]=n+1;
  for (i=k, j=n; i>0; i--)
      {sol[i] = back[i][j];
       j=sol[i]-1;}

}

void citire()
{
  int i;
  FILE * fin = fopen(InFile,"r");
  fscanf(fin,"%d %d",&n,&k);
  for(i=1;i<=n;i++)
    fscanf(fin,"%d",d+i);
  fclose(fin);
}

void afisare()
{
int i, last;
FILE * fout=fopen(OutFile,"w");
fprintf(fout,"%d\n",cmin[k][n]);
i=k;
for (i=1; i<=k; i++)
    { last=sol[i+1]-1;
      fprintf(fout,"%d\n", (sol[i]+last)/2);
    }
fclose(fout);
}
