#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define FIN "vitale.in"
#define FOUT "vitale.out"

#define MAX_N 50001
#define MAX_M 100000

typedef struct
{
    int x, y, c;
} Edge;

struct List
{
    int info;
    struct List *next;
};
typedef struct List List;

Edge *_a, *_b, E[MAX_M];
int U[MAX_M], V[MAX_M], Comp[MAX_N];
int deg[MAX_N], Lev[MAX_N], ndeg[MAX_N], *G[MAX_N], *NG[MAX_N];

int cmp(const void *a, const void *b)
{
    _a = (Edge *) a, _b = (Edge *) b;
    return _a->c - _b->c;
}

int compare(const void *a, const void *b)
{
    _a = (Edge *) a, _b = (Edge *) b;
    if (_a->x == _b->x)
       return _a->y - _b->y;

    return _a->x - _b->x;
}

void DFS(int n, int l, int f)
{
    int i, ind, x, y;

    Lev[n] = l;
    for (i = 0; i < deg[n]; i++)
    {
        ind = G[n][i];
        if (U[ind])
        {
            x = E[ind].x;
            y = E[ind].y;
            if (x != f && y != f)
               DFS(x == n ? y : x, l + 1, n);
        }
    }
}

void DF(int n, int f, List **L, List **R)
{
    int i, x, y, c, ind;
    List *start, *end, *auxl, *auxr, *p;

    start = end = NULL;

    for (i = 0; i < deg[n]; i++)
    {
        ind = G[n][i];
        x = E[ind].x;
        y = E[ind].y;
        c = E[ind].c;
        if (U[ind])
        {
            if (x != f && y != f)
            {
                DF(x != n ? x : y, n, &auxl, &auxr);
                if (!start)
                {
                    start = auxl;
                    end = auxr;
                }
                else
                {
                    end->next = auxl;
                    end = auxr;
                }
            }
        }
        else
            if (Lev[n] > Lev[x == n ? y : x])
            {
                if (!start)
                {
                    start = end = malloc(sizeof(List));
                    start->info = c;
                    start->next = NULL;
                }
                else
                {
                    end->next = malloc(sizeof(List));
                    end = end->next;
                    end->info = c;
                    end->next = NULL;
                }
            }
    }
    for (i = 0; i < deg[n]; i++)
    {
        ind = G[n][i];
        x = E[ind].x;
        y = E[ind].y;
        c = E[ind].c;
        if (V[ind])
        {
            for (p = start; p; p = p->next)
                if (p->info == c) break;
            if (p) V[ind] = 0;
        }
    }
    for (i = 0; start && i < ndeg[i]; i++)
        if (NG[n][i] == start->info) start = start->next;
        else
        {
            p = start;
            for (; p->next && p->next->info != NG[n][i]; p = p->next) ;
            if (p->next)
               p = p->next->next;
        }
    *L = start;
    *R = end;

}

int main(void)
{
    List *l, *r;
    int i, j, x, y, N, M, newname, oldname, count;

    freopen(FIN, "r", stdin);
    freopen(FOUT, "w", stdout);

    scanf("%d %d", &N, &M);
    for (i = 0; i < M; i++)
        scanf("%d %d %d", &E[i].x, &E[i].y, &E[i].c);

    /* Kruskal */
    memset(U, 0, sizeof(U));
    qsort(E, M, sizeof(Edge), cmp);
    for (i = 1; i <= N; Comp[i] = i, i++) ;
    for (i = 0; i < M; i++)
        if (Comp[E[i].x] != Comp[E[i].y])
        {
            oldname = Comp[E[i].x], newname = Comp[E[i].y];
            for (j = 1; j <= N; j++)
                if (Comp[j] == oldname)
                   Comp[j] = newname;
            U[i] = 1;
        }
    /* solve */
    memset(deg, 0, sizeof(deg));
    for (i = 0; i < M; i++)
        deg[E[i].x]++, deg[E[i].y]++;
    for (i = 1; i <= N; i++)
        G[i] = malloc(sizeof(int) * deg[i]);
    memset(deg, 0, sizeof(deg));
    for (i = 0; i < M; i++)
    {
        G[E[i].x][deg[E[i].x]++] = i;
        G[E[i].y][deg[E[i].y]++] = i;
    }
    memcpy(V, U, sizeof(U));
    DFS(1, 0, -1);
    memset(ndeg, 0, sizeof(ndeg));
    for (i = 0; i < M; i++)
        if (!U[i])
        {
            x = E[i].x;
            y = E[i].y;
            if (Lev[x] > Lev[y]) x = (x ^ y) ^ (y = x);
            if (Lev[y] - Lev[x] > 1) ndeg[x]++;
        }
    for (i = 1; i <= N; i++)
        NG[i] = malloc(sizeof(int) * ndeg[i]);
    memset(ndeg, 0, sizeof(ndeg));
    for (i = 0; i < M; i++)
        if (!U[i])
        {
            x = E[i].x;
            y = E[i].y;
            if (Lev[x] > Lev[y]) x = (x ^ y) ^ (y = x);
            if (Lev[y] - Lev[x] > 1) NG[x][ndeg[x]++] = E[i].c;
        }
    DF(1, -1, &l, &r);

    count = 0;
    for (i = 0; i < M; i++)
        if (V[i])
        {
            E[count] = E[i];
            if (E[count].x > E[count].y)
               E[count].x = (E[count].x ^ E[count].y) ^ (E[count].y = E[count].x);
            count++;
        }
    qsort(E, count, sizeof(Edge), compare);
    printf("%d\n", count);
    for (i = 0; i < count; i++)
        printf("%d %d\n", E[i].x, E[i].y);

    return 0;
}
